//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WvpXf     : WorldViewProjection   < string UIWidget="None"; >;
float4x4 WorldXf   : World                 < string UIWidget="None"; >;
float4x4 ViewIXf   : ViewInverse           < string UIWidget="None"; >;

//// TWEAKABLE PARAMETERS ////////////////////

/* Edge Scaling Factor */
float edgeScale <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 10.0;
	float  UIStep = 0.5;
	string UIName =  "Edge Scale";
> = 2;

/* Edge Scaling Factor */
float edgeThreshold <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 1.0;
	float  UIStep = 0.01;
	string UIName =  "Edge Threshold";
> = 0.2;

/* Edge Power */
float edgePower <
	string UIWidget = "slider";
	float UIMin = 1.0;
	float UIMax = 10.0;
	float UIStep = 0.5;
	string UIName = "Edge Exponent";
> = 4.0;

/// Point Lamp 0 ////////////
float3 Lamp0Pos : Position <
    string Object = "PointLight0";
    string UIName =  "Lamp 0 Position";
    string Space = "World";
> = {-0.5f,2.0f,1.25f};

float3 Lamp0Color : Specular <
    string UIName =  "Lamp 0";
    string Object = "Pointlight0";
    string UIWidget = "Color";
> = {1.0f,1.0f,1.0f};


/* Spekularer Exponent */
float shininess : SpecularPower <
	string UIWidget = "slider";
	float UIMin = 1.0;
	float UIMax = 1000.0;
	float UIStep = 10.0;
	string UIName = "Specular Power";
> = 30.0;


/* Farbe der Oberfläche */
float3 DiffuseColor0  <
	string UIName =  "Diffuse Color 0";
	string UIWidget = "Color";
> = {0.8f, 0.8f, 1.0f};

float3 DiffuseColor1 <
	string UIName =  "Diffuse Color 1";
	string UIWidget = "Color";
> = {0.8f, 0.8f, 1.0f};

float3 SpecularColor0 <
	string UIName =  "Specular Color 0";
	string UIWidget = "Color";
> = {0.8f, 0.8f, 1.0f};

float3 SpecularColor1 <
	string UIName =  "Specular Color 1";
	string UIWidget = "Color";
> = {0.8f, 0.8f, 1.0f};

/* Diffuse Threshold */
float diffThreshold <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 1.0;
	float  UIStep = 0.01;
	string UIName =  "Diffuse Threshold";
> = 0.5;

/* Speacular Factor */
float specThreshold <
	string UIWidget = "slider";
	float  UIMin = 0.0;
	float  UIMax = 1.0;
	float  UIStep = 0.01;
	string UIName =  "Specular Threshold";
> = 0.5;

float texScale <
	string UIWidget = "slider";
    float uimin = 0;
    float uimax = 30.0;
	float  UIStep = 0.1;
    string Desc = "TAM: Texture Scale";
> = 15;

float threshold0 <
	string UIWidget = "slider";
    float uimin = 0;
    float uimax = 1;
	float  UIStep = 0.01;
    string Desc = "TAM: Threshold 0";
> = 0.2;
float threshold1 <
	string UIWidget = "slider";
    float uimin = 0;
    float uimax = 1;
	float  UIStep = 0.01;
    string Desc = "TAM: Threshold 1";
> = 0.4;

float threshold2 <
	string UIWidget = "slider";
    float uimin = 0;
    float uimax = 1;
	float  UIStep = 0.01;
    string Desc = "TAM: Threshold 2";
> = 0.6;

float threshold3 <
	string UIWidget = "slider";
    float uimin = 0;
    float uimax = 1;
	float  UIStep = 0.01;
    string Desc = "TAM: Threshold 3";
> = 0.8;

float threshold4 <
	string UIWidget = "slider";
    float uimin = 0;
    float uimax = 1;
	float  UIStep = 0.01;
    string Desc = "TAM: Threshold 4";
> = 0.9;

float threshold5 <
	string UIWidget = "slider";
    float uimin = 0;
    float uimax = 1;
	float  UIStep = 0.01;
    string Desc = "TAM: Threshold 5";
> = 1.0;


texture ColorMap <
    string ResourceName = "ColorShades.dds";
    string UIName =  "Color Table";
    string ResourceType = "1D";
    
>;
sampler ColorMapSampler = sampler_state 
{
    texture = <ColorMap>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
	Filter    = MIN_MAG_MIP_LINEAR;
};


texture StrokeMapA <
    string ResourceName = "StrokesA_MIP.dds";
    string UIName       =  "Strokes A";
    string ResourceType = "2D";
    
>;

texture StrokeMapB <
    string ResourceName = "StrokesB_MIP.dds";
    string UIName       = "Strokes B";
    string ResourceType = "2D";    
>;

sampler StrokeMapASampler = sampler_state 
{
    texture = <StrokeMapA>;
    AddressU  = Wrap;        
    AddressV  = Wrap;
    AddressW  = Wrap;
	Filter    = MIN_MAG_MIP_LINEAR;
};

sampler StrokeMapBSampler = sampler_state 
{
    texture = <StrokeMapB>;
    AddressU  = Wrap;        
    AddressV  = Wrap;
    AddressW  = Wrap;
	Filter    = MIN_MAG_MIP_LINEAR;
};


//////// CONNECTOR DATA STRUCTURES ///////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
    float4 Binormal	: BINORMAL0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		: TEXCOORD0;
    // The following values are passed in "World" coordinates since
    //   it tends to be the most flexible and easy for handling
    //   reflections, sky lighting, and other "global" effects.
    float3 LightVec	: TEXCOORD1;
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldTangent	: TEXCOORD3;
    float3 WorldBinormal : TEXCOORD4;
    float3 WorldView	: TEXCOORD5;
};
 
///////// VERTEX SHADING /////////////////////

/*********** Generic Vertex Shader ******/

vertexOutput std_VS(appdata IN) {
    vertexOutput OUT = (vertexOutput)0;
    OUT.WorldNormal = mul(IN.Normal,WorldITXf).xyz;
    OUT.WorldTangent = mul(IN.Tangent,WorldITXf).xyz;
    OUT.WorldBinormal = mul(IN.Binormal,WorldITXf).xyz;
    float4 Po = float4(IN.Position.xyz,1);
    float3 Pw = mul(Po,WorldXf).xyz;
    OUT.LightVec = (Lamp0Pos - Pw);
    OUT.UV = IN.UV.xy;
    OUT.WorldView = normalize(ViewIXf[3].xyz - Pw);
    OUT.HPosition = mul(Po,WvpXf);
    return OUT;
}

///////// PIXEL SHADING //////////////////////

float4 silhouette_PS(vertexOutput IN) : COLOR {
    float3 Vn = normalize(IN.WorldView);
    float3 Nn = normalize(IN.WorldNormal);
	
  	// Perform edge detection
  	float edge = edgeScale*pow(dot(Vn, Nn),edgePower);
  	edge = step(edgeThreshold,edge);

    return float4(edge.xxx,1.0);
}

float4 toon_PS(vertexOutput IN) : COLOR {
    
	float3 Vn = normalize(IN.WorldView);
    float3 Nn = normalize(IN.WorldNormal);
	float3 Ln = normalize(IN.LightVec);
   	float3 Hn = normalize(Ln + Vn);
   
   	float diff =dot(Ln, Nn);
   	float spec = pow(max(0, dot(Hn, Nn)), shininess);

  // Perform edge detection
  float edge = edgeScale*pow(dot(Vn, Nn),edgePower);
  edge = step(edgeThreshold,edge);

  float3 color;
  
  // light terms are thresholded to generate only two disctinct tones respectively. 
  float3 diffColor = (diff > diffThreshold)? DiffuseColor1 : DiffuseColor0;
  float3 specColor = (spec > specThreshold)? SpecularColor1 : SpecularColor0;
  color = edge* (diffColor+specColor);

   return float4(color,1.0);
}

float4 colortable_PS(vertexOutput IN) : COLOR {
    float3 Vn = normalize(IN.WorldView);
    float3 Ln = normalize(IN.LightVec);
    float3 Nn = normalize(IN.WorldNormal);
	
  	// Perform edge detection
  	float edge = edgeScale*pow(dot(Vn, Nn),edgePower);
  	edge = step(edgeThreshold,edge);

  	float3 color;
  
    float diff = 0.5*(1+dot(Ln, Nn));
  	// diffuse term is used to interpolate between cool and warm color
 	color.xyz = edge * tex1D(ColorMapSampler,diff).rgb;
  	

    return float4(color,1.0);
}


float4 tonalArtMap_PS(vertexOutput IN) : COLOR {
    float3 Vn = normalize(IN.WorldView);
    float3 Nn = normalize(IN.WorldNormal);
   float3 Ln = normalize(IN.LightVec);
   float3 Hn = normalize(Ln + Vn);
   
   float diff = (dot(Ln, Nn));
   float spec = pow(max(0, dot(Hn, Nn)), shininess);
   
  	// Perform edge detection
  	float edge = edgeScale*pow(dot(Vn, Nn),edgePower);
  	edge = step(edgeThreshold,edge);

  float4 color;
  
  // diffuse term is used to interpolate between cool and warm color
  color.xyz = edge * tex2D(StrokeMapASampler,IN.UV).rgb;
  color.w = 1;
 
   // texture coordinates for stroke textures are specified with
   // respect to the object space
   float3 strokes1 = tex2D(StrokeMapASampler,texScale*IN.UV).xyz;
   float3 strokes2 = tex2D(StrokeMapBSampler,texScale*IN.UV).xyz;

  float3 stroke = float3(1,1,1);
  float alpha;
  float light = diff+spec;

  // use light intensity to interpolate between different 
  // levels of stroke textures
  if (light < threshold0) {
        alpha = (light)/(threshold0);
	stroke = lerp(strokes2.zzz,strokes2.yyy,alpha);

  } else if (light < threshold1) {
        alpha = (light-threshold0)/(threshold1-threshold0);
	stroke = lerp(strokes2.yyy,strokes2.xxx,alpha);

  } else if (light < threshold2) {
        alpha = (light-threshold1)/(threshold2-threshold1);
	stroke = lerp(strokes2.xxx,strokes1.zzz,alpha);

  } else if (light < threshold3) {
        alpha = (light-threshold2)/(threshold3-threshold2);
	stroke = lerp(strokes1.zzz,strokes1.yyy,alpha);

  } else if (light < threshold4) {
        alpha = (light-threshold3)/(threshold4-threshold3);
	stroke = lerp(strokes1.yyy,strokes1.xxx,alpha);

  } else if (light < threshold5){
        alpha = (light-threshold4)/(threshold5-threshold4);
	stroke = lerp(strokes1.xxx,float3(1,1,1),alpha);
  } else {
	stroke = float3(1,1,1);
  }
  color.xyz =  edge * stroke.xxx;
  color.w = 1;

   return color;

    return float4(edge.xxx,1.0);
}


///// TECHNIQUES /////////////////////////////

RasterizerState DisableCulling
{
    CullMode = NONE;
};

DepthStencilState DepthEnabling
{
	DepthEnable = TRUE;
};

BlendState DisableBlend
{
	BlendEnable[0] = FALSE;
};

technique10 Silhouette10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, silhouette_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}

technique10 ToonShading10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, toon_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}

technique10 ColorTable10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, colortable_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}


technique10 TonalArtMaps10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, tonalArtMap_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}



/////////////////////////////////////// eof //
